﻿/*
  DXL APIを使う
   複数軸への角速度・角度指令
*/

#include  <stdio.h>
#include  <conio.h>
#include  <process.h>
#include  "dx2lib.h"

#define COMPORT     "\\\\.\\COM5"   // ポート番号
#define BAUDRATE    (57600)         // ボーレート[bps]

#ifndef _WIN32
#include <unistd.h>
#define Sleep(w)  usleep(w*1000)
#endif

//-----------------------------------------------------------
// 予約語
//-----------------------------------------------------------
// スレッド伝達用構造体
typedef struct {
  TDeviceID dev;      // デバイスID
  const uint8_t *ids; // IDの配列
  int num;            // ID数
} TMonInfo;

//-----------------------------------------------------------
// グローバル変数
//-----------------------------------------------------------
bool term = false;  // スレッド終了フラグ

//===========================================================
// 複数軸の現在角度・角速度・エラー情報をモニタするスレッド
//===========================================================
unsigned __stdcall monthread (void *pdev) {
  TMonInfo *info = (TMonInfo *)pdev;
  double  pangle[info->num], pvelo[info->num], pcur[info->num];
  for (int i = 0; i < info->num; i++) pangle[i] = pvelo[i] = pcur[i] = 0.0;
  while (!term) {
    // 現在位置取得
    DXL_GetPresentAngles (info->dev, info->ids, pangle, info->num);
    // 現在角速度取得
    DXL_GetPresentVelocities (info->dev, info->ids, pvelo, info->num);
    // 現在電流取得
    DXL_GetPresentCurrents(info->dev, info->ids, pcur, info->num);
    for (int i = 0; i < info->num; i++)
      printf ("(%d:$%04X %4.0f%6.1f)", info->ids[i], DXL_GetErrorCode (info->dev, info->ids[i]), pangle[i], pvelo[i]);
    printf("\r");
    Sleep (10);
  }
  _endthreadex (0);
  return 0;
}

//===========================================================
// メイン
//===========================================================
void main (void) {
  TMonInfo info;
  HANDLE th;

  // 検出したDynamixelのID一覧
  uint8_t ids[253];
  // 検出したDynamixel数
  int num = 0;

  TDeviceID dev = DX2_OpenPort (COMPORT, BAUDRATE);
  if (dev != 0) {
    printf ("Open success\n");

    // 接続された全デバイスを検索しその一覧を表示
    printf ("scanning devices...");
    num = DXL_ScanDevices (dev, ids);
    printf (" %d device detected.\n", num);
    DXL_PrintDevicesList ((void *)&printf);

    // モニタスレッド開始
    info.dev = dev;
    info.ids = ids;
    info.num = num;
    th = (HANDLE)_beginthreadex (NULL, 0, &monthread, (void *)&info, 0, NULL);

    // 角度・角速度同時指令用変数
    TAngleVelocity AngleVelocity[num];
    for (int i = 0; i < num; i++) AngleVelocity[i].velocity = 45; // 45deg/s

    // Positonモードに設定
    if (DXL_SetOperatingModesEquival (dev, ids, num, 3)) {
      // 制御開始
      DXL_SetTorqueEnablesEquival (dev, ids, num, true);

      for (int i = 0; i < num; i++) AngleVelocity[i].angle = 90;   // 90deg
      DXL_SetGoalAnglesAndVelocities (dev, ids, AngleVelocity, num);
      Sleep (3000);
      for (int i = 0; i < num; i++) AngleVelocity[i].angle = 0;    // 0deg
      DXL_SetGoalAnglesAndVelocities (dev, ids, AngleVelocity, num);
      Sleep (3000);
      for (int i = 0; i < num; i++) AngleVelocity[i].angle = -90;  // -90deg
      DXL_SetGoalAnglesAndVelocities (dev, ids, AngleVelocity, num);
      Sleep (3000);
      for (int i = 0; i < num; i++) AngleVelocity[i].angle = 0;    // 0deg
      DXL_SetGoalAnglesAndVelocities (dev, ids, AngleVelocity, num);
      Sleep (3000);

      // 制御停止
      DXL_SetTorqueEnablesEquival (dev, ids, num, false);
    }
    // モニタスレッド終了
    term = true;
    WaitForSingleObject (th, 2000);
    CloseHandle (th);

    DX2_ClosePort (dev);
  }
}
