﻿/*
  BULKインストラクションで複数台から一括で読み書き

  ターゲット:X
*/

#include  <stdio.h>
#include  <conio.h>
#include  "dx2lib.h"
#include  "dx2memmap.h"

// マクロ定義
#define _COMPORT  (5)       // デフォルトポート番号
#define _BAUDRATE (57600)   // デフォルトボーレート[bps]
#define _TARGETID (1)       // デフォルト先頭ID

#define MAXPOS    (4095)
#define MINPOS    (0)
#define DIFFPOS   (2)

#define AXISNUM   (2)   // 対象となる軸数(_TARGETID=1だとしたら2,3,4...と連続しているものとし全軸存在している必要がある

// 現在位置をREAD BULKで読み出した際のデータ用構造体 (packed必須)
// 本来可変長のデータだが、ここでは各軸から同じアドレス・データ長を指定して取得するので固定化
typedef struct {
  uint16_t size;  // 1軸分のデータサイズ
  uint8_t  id;    // id
  uint16_t err;   // ステータス
  int32_t  ppos;  // データ
} __attribute__ ((gcc_struct, __packed__)) TBRData_PPos[AXISNUM];

// 目標位置をWRITE BULKで書き込む際のパラメータ用構造体 (packed必須)
typedef struct {
  uint8_t  id;    // id
  uint16_t addr;  // アドレス
  uint16_t len;   // データ長
  int32_t  pos;   // データ
} __attribute__ ((gcc_struct, __packed__)) TBWParam_Pos[AXISNUM];

//===========================================================
// 実行時にポート番号・ボーレート・IDを選択
//===========================================================
bool InputNum (int32_t *num) {
  char buf[100];
  fgets(buf, sizeof(buf), stdin);
  rewind(stdin);
  return (strlen(buf) && sscanf(buf, "%d", num) == 1);
}

bool EditComAndBaudAndTarget (char *comport, uint32_t *comn, uint32_t *baud, uint8_t *id) {
  bool result = true;

  int32_t n;

  printf ("Input comport num (default %d) = ", _COMPORT);
  if (!InputNum (&n)) { n = _COMPORT; result = false; }
  if (n >= 1 && n <= 255) *comn = n; else *comn = _COMPORT;
  sprintf(comport, "\\\\.\\COM%d", *comn);

  printf ("Input baudrate (default %dbps) = ", _BAUDRATE);
  if (!InputNum (&n)) { n = _BAUDRATE; result = false; }
  if (n >= 9600 && n <= 3000000) *baud = n; else *baud = _BAUDRATE;

  printf ("Input target id (default %d) = ", _TARGETID);
  if (!InputNum (&n)) { n = _TARGETID; result = false; }
  if (n >= 1 && n < 0xfd) *id = n; else *id = _TARGETID;

  return result;
}

//===========================================================
// モデル番号からデバイス名を特定
//===========================================================
TDXL_DevType CheckModelInfo (TDeviceID dev, uint8_t id) {
  uint16_t modelno;
  if (DX2_ReadWordData (dev, id, 0, &modelno, NULL)) {
    switch (modelno) {
      case 0x015E:  // XL-320
        return devtXL320;

      case 0x001E:  // MX-28(2.0)
      case 0x0137:  // MX-64(2.0)
      case 0x0141:  // MX-106(2.0)
      case 0x04A6:  // XL330-M077
      case 0x04B0:  // XL330-M288
      case 0x0424:  // XL430-W250
      case 0x0442:  // 2XL430-W250
      case 0x0488:  // 2XC430-W250
      case 0x042E:  // XC430-W150
      case 0x0438:  // XC430-W240
      case 0x0406:  // XM430-W210
      case 0x03F2:  // XH430-W210
      case 0x041A:  // XH430-V210
      case 0x03FC:  // XM430-W350
      case 0x03E8:  // XH430-W350
      case 0x0410:  // XH430-V350
      case 0x046A:  // XM540-W150
      case 0x0456:  // XH540-W150
      case 0x047E:  // XH540-V150
      case 0x0460:  // XM540-W270
      case 0x044C:  // XH540-W270
      case 0x0474:  // XH540-V270
      case 0x049C:  // XW540-T140
      case 0x0492:  // XW540-T260
        return devtX;

      case 0xD308:  // H54-200-S500-R
      case 0xD208:  // H54-100-S500-R
      case 0xC800:  // H42-20-S300-R
      case 0xB510:  // M54-60-S250-R
      case 0xB410:  // M54-40-S250-R
      case 0xA918:  // M42-10-S260-R
      case 0x9520:  // L54-50-S290-R
      case 0x9508:  // L54-50-S500-R
      case 0x9428:  // L54-30-S400-R
      case 0x9408:  // L54-30-S500-R
      case 0x8900:  // L42-10-S300-R
        return devtPRO;

      case 0xD309:  // H54-200-S500-RA
      case 0xD209:  // H54-100-S500-RA
      case 0xC801:  // H42-20-S300-RA
      case 0xB511:  // M54-60-S250-RA
      case 0xB411:  // M54-40-S250-RA
      case 0xA919:  // M42-10-S260-RA
      case 0x07E4:  // H54P-200-S500-R
      case 0x07DA:  // H54P-100-S500-R
      case 0x07D0:  // H42P-020-S300-R
      case 0x0848:  // M54P-60-S250-R
      case 0x083E:  // M54P-40-S250-R
      case 0x0834:  // M42P-10-S260-R
        return devtPROP;
      // その他
      default:
        return devtNONE;
    }
  }
  return devtNONE;
}

//===========================================================
// メイン
//===========================================================
void main (void) {
  TDeviceID   dev;
  TErrorCode  err;

  // READ BULKで現在位置を取得するパラメータ
  TBulkReadParam BRP_Ppos[AXISNUM];
  for (int i = 0; i < AXISNUM; i++) {
    BRP_Ppos[i].id     = i + _TARGETID;
    BRP_Ppos[i].addr   = ADDRESS_X_PRESENT_POSITION;
    BRP_Ppos[i].length = sizeof (int32_t);
  };
  // READ BULKで取得した現在位置
  TBRData_PPos BR_PPos;

  // WRITE BULKで位置を指令するパラメータ
  TBWParam_Pos BWP_Pos;
  for (int i = 0; i < AXISNUM; i++) {
    BWP_Pos[i].id   = i + _TARGETID;
    BWP_Pos[i].addr = ADDRESS_X_GOAL_POSITION;
    BWP_Pos[i].len  = sizeof (int32_t);
  }

  // 目標位置指令毎の増分
  int32_t diff[AXISNUM];
  for (int i = 0; i < AXISNUM; i++) diff[i] = DIFFPOS;

  char comname[20];
  uint32_t comnum, baud;
  uint8_t id;

  EditComAndBaudAndTarget (comname, &comnum, &baud, &id);
  printf("\nCOM=%s, Baudrate=%d, id=%d\n", comname, baud, id);

  if ((dev = DX2_OpenPort (comname, baud))) {
    printf ("Open success\n");

    // 全軸がXシリーズである事を確認
    bool pass = true;
    for (int i = 0; i < AXISNUM; i++) pass = pass && (CheckModelInfo (dev, BRP_Ppos[i].id) == devtX);

    if (pass) {
      // 現在位置を取得
      uint32_t num = AXISNUM;
      if (DX2_ReadBulkData (dev, &BRP_Ppos[0], &num, (uint8_t *)&BR_PPos, &err)) {
        printf ("present pos = ");
        for (int i = 0; i < AXISNUM; i++) {
          BWP_Pos[i].pos = BR_PPos[i].ppos; // 開始位置を現在位置に
          printf ("[%d]%4d%s", BR_PPos[i].id, BR_PPos[i].ppos, i == AXISNUM - 1 ? "\n\n" : " ");
        }

        // トルクイネーブル
        DX2_WriteByteData (dev, BROADCASTING_ID, ADDRESS_X_TORQUE_ENABLE, 1, &err);

        // 何かキーを押すとループを抜ける
        while (!kbhit() && DX2_Active (dev)) {
          for (int i = 0; i < AXISNUM; i++) {
            BWP_Pos[i].pos += diff[i];
            if (BWP_Pos[i].pos >= MAXPOS) {
              BWP_Pos[i].pos = MAXPOS;
              diff[i] *= -1;
            } else if (BWP_Pos[i].pos <= MINPOS) {
              BWP_Pos[i].pos = MINPOS;
              diff[i] *= -1;
            }
            printf ("[%d]%4d%s", BWP_Pos[i].id, BWP_Pos[i].pos, i == AXISNUM - 1 ? "\r" : " ");
          }
          // 目標角度指令
          DX2_WriteBulkData (dev, (uint8_t *)&BWP_Pos, sizeof (BWP_Pos), &err);

          // 1ミリ秒待ち
          Sleep (1);
        }

        // トルクディスエーブル
        DX2_WriteByteData (dev, BROADCASTING_ID, ADDRESS_X_TORQUE_ENABLE, 0, &err);
      } else printf ("ReadBulk Error [%04X]\n", err);
    }

    DX2_ClosePort (dev);
  } else printf ("Open error\n");

  printf ("\nFin");
}
