﻿/*
  DXL APIを使う
   角度指令
*/

#include <stdio.h>
#include "dx2lib.h"

#define COMPORT   "/dev/ttyUSB0"  // ポート名
#define BAUDRATE  (57600)         // ボーレート[bps]
#define TARGETID  (1)             // 対象ID

//===========================================================
// ミリ秒スリープ
//===========================================================
void Sleep (int milliseconds) {
  struct timespec ts;
  ts.tv_sec = milliseconds / 1000;
  ts.tv_nsec = (milliseconds % 1000) * 1000000;
  nanosleep(&ts, NULL);
}

//===========================================================
// メイン
//===========================================================
int main (void) {
  TDeviceID dev = DX2_OpenPort (COMPORT, BAUDRATE);

  if (dev != 0) {
    printf ("Open success\n");

    // 指定IDのデバイス情報を取得
    // モデルNo.が既存のDynamixelと一致しないと以後機能しない
    printf ("[id:%d] Model Name=%s\n", TARGETID, DXL_GetModelInfo (dev, TARGETID)->name);

    // マルチターンJointモードに設定
    if (DXL_SetOperatingMode (dev, TARGETID, 4)) {
      // LED点灯
      DXL_SetLED (dev, TARGETID, true);
      // トルクイネーブル
      DXL_SetTorqueEnable (dev, TARGETID, true);

      DXL_SetGoalAngle (dev, TARGETID, 720.0);   // 720deg
      Sleep (5000);
      DXL_SetGoalAngle (dev, TARGETID, -720.0);  // -720deg
      Sleep (10000);
      DXL_SetGoalAngle (dev, TARGETID, 0.0);    // 0deg
      Sleep (5000);

      // トルクディスエーブル
      DXL_SetTorqueEnable (dev, TARGETID, false);
      // LED消灯
      DXL_SetLED (dev, TARGETID, false);
    } else {
      printf ("SetOPmode error\n");
    }
    DX2_ClosePort (dev);
  } else {
    printf ("Open error\n");
  }
}
