﻿/*
  8bit幅のアイテムへのアクセス

  ターゲット:PRO+, PRO, X, MX, XL320
*/

#include  <stdio.h>
#include  "dx2lib.h"
// デバイスのアイテムアドレスの一部を宣言
#include  "dx2memmap.h"

// マクロ定義
#define COMPORT   "/dev/ttyUSB0"  // ポート名
#define BAUDRATE  (57600)         // ボーレート[bps]
#define _TARGETID (1)             // デフォルト対象ID

//===========================================================
// ミリ秒スリープ
//===========================================================
void Sleep (int milliseconds) {
  struct timespec ts;
  ts.tv_sec = milliseconds / 1000;
  ts.tv_nsec = (milliseconds % 1000) * 1000000;
  nanosleep(&ts, NULL);
}

//===========================================================
// モデル番号からデバイス名を特定
//===========================================================
TDXL_DevType CheckModelInfo (TDeviceID dev, uint8_t id) {
  uint16_t modelno;
  if (DX2_ReadWordData (dev, id, 0, &modelno, NULL)) {
    switch (modelno) {
      case 0x015E:  // XL-320
        return devtXL320;

      case 0x001E:  // MX-28(2.0)
      case 0x0137:  // MX-64(2.0)
      case 0x0141:  // MX-106(2.0)
      case 0x04A6:  // XL330-M077
      case 0x04B0:  // XL330-M288
      case 0x0424:  // XL430-W250
      case 0x0442:  // 2XL430-W250
      case 0x0488:  // 2XC430-W250
      case 0x042E:  // XC430-W150
      case 0x0438:  // XC430-W240
      case 0x0406:  // XM430-W210
      case 0x03F2:  // XH430-W210
      case 0x041A:  // XH430-V210
      case 0x03FC:  // XM430-W350
      case 0x03E8:  // XH430-W350
      case 0x0410:  // XH430-V350
      case 0x046A:  // XM540-W150
      case 0x0456:  // XH540-W150
      case 0x047E:  // XH540-V150
      case 0x0460:  // XM540-W270
      case 0x044C:  // XH540-W270
      case 0x0474:  // XH540-V270
      case 0x049C:  // XW540-T140
      case 0x0492:  // XW540-T260
        return devtX;

      case 0xD308:  // H54-200-S500-R
      case 0xD208:  // H54-100-S500-R
      case 0xC800:  // H42-20-S300-R
      case 0xB510:  // M54-60-S250-R
      case 0xB410:  // M54-40-S250-R
      case 0xA918:  // M42-10-S260-R
      case 0x9520:  // L54-50-S290-R
      case 0x9508:  // L54-50-S500-R
      case 0x9428:  // L54-30-S400-R
      case 0x9408:  // L54-30-S500-R
      case 0x8900:  // L42-10-S300-R
        return devtPRO;

      case 0xD309:  // H54-200-S500-RA
      case 0xD209:  // H54-100-S500-RA
      case 0xC801:  // H42-20-S300-RA
      case 0xB511:  // M54-60-S250-RA
      case 0xB411:  // M54-40-S250-RA
      case 0xA919:  // M42-10-S260-RA
      case 0x07E4:  // H54P-200-S500-R
      case 0x07DA:  // H54P-100-S500-R
      case 0x07D0:  // H42P-020-S300-R
      case 0x0848:  // M54P-60-S250-R
      case 0x083E:  // M54P-40-S250-R
      case 0x0834:  // M42P-10-S260-R
        return devtPROP;
      // その他
      default:
        return devtNONE;
    }
  }
  return devtNONE;
}

//===========================================================
// メイン
//===========================================================
int main (void) {
  uint8_t     led;
  TDeviceID   dev;
  TErrorCode  err;

  uint8_t id = _TARGETID;

  // ポートオープン
  if ((dev = DX2_OpenPort (COMPORT, BAUDRATE))) {
    printf ("Open success\n");

    uint16_t addr_led;
    // PINGで指定IDの存在を確認(モデルまでは判断していないので注意)
    switch (CheckModelInfo (dev, id)) {
      case devtXL320: addr_led = 25; break;
      case devtX: addr_led = ADDRESS_X_LED_RED; break;
      case devtPRO: addr_led = ADDRESS_PRO_LED_RED; break;
      case devtPROP: addr_led = ADDRESS_PROP_LED_RED; break;
      default: addr_led = 0; break;
    }
    if (addr_led != 0) {
      for (int i = 0; i < 10; i++) {
        // LEDの状態を読み出し
        if (DX2_ReadByteData (dev, id, addr_led, &led, &err))
          printf ("Read ok led=%d\n", led);
        else
          printf ("Read ng [$%04X]\n", err);

        // 読み出したLEDの値を反転
        if (led == 0) led = 255;
        else led = 0;

        // LEDへ値を書き込み
        if (DX2_WriteByteData (dev, id, addr_led, led, &err))
          printf ("Write ok led=%d\n", led);
        else
          printf ("Write ng [$%04X]\n", err);

        // 0.5秒待ち
        Sleep (500);
      }
    } else printf ("Not supported device ID=%d\n", id);

    // ポートクローズ
    DX2_ClosePort (dev);
  } else
    printf ("Open error\n");

  printf ("Fin\n");
}
